@@ -27,6 +27,7 @@
 
 #include "NetworkManager.h"
 #include "nm-device-ethernet.h"
+#include "nm-device-adsl.h"
 #include "nm-device-wifi.h"
 #include "nm-device-modem.h"
 #include "nm-device-bt.h"
@@ -34,11 +35,14 @@
 #include "nm-device-wimax.h"
 #include "nm-device-infiniband.h"
 #include "nm-device-bond.h"
+#include "nm-device-bridge.h"
 #include "nm-device-vlan.h"
 #include "nm-device.h"
 #include "nm-device-private.h"
 #include "nm-object-private.h"
 #include "nm-object-cache.h"
+#include "nm-remote-connection.h"
+#include "nm-types.h"
 #include "nm-glib-marshal.h"
 #include "nm-dbus-glib-types.h"
 #include "nm-glib-compat.h"
@@ -60,7 +64,6 @@ G_DEFINE_TYPE_WITH_CODE (NMDevice, nm_device, NM_TYPE_OBJECT,
 #define NM_DEVICE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), NM_TYPE_DEVICE, NMDevicePrivate))
 
 typedef struct {
-	gboolean disposed;
 	DBusGProxy *proxy;
 
 	char *iface;
@@ -68,17 +71,22 @@ typedef struct {
 	NMDeviceType device_type;
 	char *udi;
 	char *driver;
+	char *driver_version;
+	char *firmware_version;
 	NMDeviceCapabilities capabilities;
 	gboolean managed;
 	gboolean firmware_missing;
+	gboolean autoconnect;
 	NMIP4Config *ip4_config;
 	NMDHCP4Config *dhcp4_config;
 	NMIP6Config *ip6_config;
 	NMDHCP6Config *dhcp6_config;
 	NMDeviceState state;
+	NMDeviceState last_seen_state;
 	NMDeviceStateReason reason;
 
 	NMActiveConnection *active_connection;
+	GPtrArray *available_connections;
 
 	GUdevClient *client;
 	char *product;
@@ -90,8 +98,11 @@ enum {
 	PROP_INTERFACE,
 	PROP_UDI,
 	PROP_DRIVER,
+	PROP_DRIVER_VERSION,
+	PROP_FIRMWARE_VERSION,
 	PROP_CAPABILITIES,
 	PROP_MANAGED,
+	PROP_AUTOCONNECT,
 	PROP_FIRMWARE_MISSING,
 	PROP_IP4_CONFIG,
 	PROP_DHCP4_CONFIG,
@@ -104,6 +115,7 @@ enum {
 	PROP_IP_INTERFACE,
 	PROP_DEVICE_TYPE,
 	PROP_ACTIVE_CONNECTION,
+	PROP_AVAILABLE_CONNECTIONS,
 
 	LAST_PROP
 };
@@ -152,8 +164,11 @@ register_properties (NMDevice *device)
 		{ NM_DEVICE_INTERFACE,         &priv->iface },
 		{ NM_DEVICE_IP_INTERFACE,      &priv->ip_iface },
 		{ NM_DEVICE_DRIVER,            &priv->driver },
+		{ NM_DEVICE_DRIVER_VERSION,    &priv->driver_version },
+		{ NM_DEVICE_FIRMWARE_VERSION,  &priv->firmware_version },
 		{ NM_DEVICE_CAPABILITIES,      &priv->capabilities },
 		{ NM_DEVICE_MANAGED,           &priv->managed },
+		{ NM_DEVICE_AUTOCONNECT,       &priv->autoconnect },
 		{ NM_DEVICE_FIRMWARE_MISSING,  &priv->firmware_missing },
 		{ NM_DEVICE_IP4_CONFIG,        &priv->ip4_config, NULL, NM_TYPE_IP4_CONFIG },
 		{ NM_DEVICE_DHCP4_CONFIG,      &priv->dhcp4_config, NULL, NM_TYPE_DHCP4_CONFIG },
@@ -162,6 +177,7 @@ register_properties (NMDevice *device)
 		{ NM_DEVICE_STATE,             &priv->state },
 		{ NM_DEVICE_STATE_REASON,      &priv->state, demarshal_state_reason },
 		{ NM_DEVICE_ACTIVE_CONNECTION, &priv->active_connection, NULL, NM_TYPE_ACTIVE_CONNECTION },
+		{ NM_DEVICE_AVAILABLE_CONNECTIONS, &priv->available_connections, NULL, NM_TYPE_REMOTE_CONNECTION },
 
 		/* Properties that exist in D-Bus but that we don't track */
 		{ "ip4-address", NULL },
@@ -175,6 +191,45 @@ register_properties (NMDevice *device)
 	                                property_info);
 }
 
+typedef struct {
+	NMDeviceState old_state;
+	NMDeviceState new_state;
+	NMDeviceStateReason reason;
+} StateChangeData;
+
+static void
+device_state_change_reloaded (GObject *object,
+                              GAsyncResult *result,
+                              gpointer user_data)
+{
+	NMDevice *self = NM_DEVICE (object);
+	NMDevicePrivate *priv = NM_DEVICE_GET_PRIVATE (self);
+	StateChangeData *data = user_data;
+	NMDeviceState old_state = data->old_state;
+	NMDeviceState new_state = data->new_state;
+	NMDeviceStateReason reason = data->reason;
+
+	g_slice_free (StateChangeData, data);
+
+	_nm_object_reload_properties_finish (NM_OBJECT (object), result, NULL);
+
+	/* If the device changes state several times in rapid succession, then we'll
+	 * queue several reload_properties() calls, and there's no guarantee that
+	 * they'll finish in the right order. In that case, only emit the signal
+	 * for the last one.
+	 */
+	if (priv->last_seen_state != new_state)
+		return;
+
+	/* Ensure that nm_device_get_state() will return the right value even if
+	 * we haven't processed the corresponding PropertiesChanged yet.
+	 */
+	priv->state = new_state;
+
+	g_signal_emit (self, signals[STATE_CHANGED], 0,
+	               new_state, old_state, reason);
+}
+
 static void
 device_state_changed (DBusGProxy *proxy,
                       NMDeviceState new_state,
@@ -183,16 +238,25 @@ device_state_changed (DBusGProxy *proxy,
                       gpointer user_data)
 {
 	NMDevice *self = NM_DEVICE (user_data);
+	NMDevicePrivate *priv = NM_DEVICE_GET_PRIVATE (self);
 
 	if (old_state != new_state) {
-		/* Update state here since the PropertyChanged signal for state
-		 * might come in a bit later, but a client might ask for the
-		 * state via nm_device_get_state() as a result of this signal.
-		 * When the PC signal does come in that will trigger the glib
-		 * property notify signal so we don't need to do that here.
+		StateChangeData *data;
+
+		/* Our object-valued properties (eg, ip4_config) will still
+		 * have their old values at this point, because NMObject is
+		 * in the process of asynchronously reading the new values.
+		 * Wait for that to finish before emitting the signal.
 		 */
-		NM_DEVICE_GET_PRIVATE (self)->state = new_state;
-		g_signal_emit (self, signals[STATE_CHANGED], 0, new_state, old_state, reason);
+		priv->last_seen_state = new_state;
+
+		data = g_slice_new (StateChangeData);
+		data->old_state = old_state;
+		data->new_state = new_state;
+		data->reason = reason;
+		_nm_object_reload_properties_async (NM_OBJECT (user_data),
+		                                    device_state_change_reloaded,
+		                                    data);
 	}
 }
 
@@ -208,6 +272,8 @@ _nm_device_gtype_from_dtype (NMDeviceType dtype)
 		return NM_TYPE_DEVICE_MODEM;
 	case NM_DEVICE_TYPE_BT:
 		return NM_TYPE_DEVICE_BT;
+	case NM_DEVICE_TYPE_ADSL:
+		return NM_TYPE_DEVICE_ADSL;
 	case NM_DEVICE_TYPE_OLPC_MESH:
 		return NM_TYPE_DEVICE_OLPC_MESH;
 	case NM_DEVICE_TYPE_WIMAX:
@@ -216,6 +282,8 @@ _nm_device_gtype_from_dtype (NMDeviceType dtype)
 		return NM_TYPE_DEVICE_INFINIBAND;
 	case NM_DEVICE_TYPE_BOND:
 		return NM_TYPE_DEVICE_BOND;
+	case NM_DEVICE_TYPE_BRIDGE:
+		return NM_TYPE_DEVICE_BRIDGE;
 	case NM_DEVICE_TYPE_VLAN:
 		return NM_TYPE_DEVICE_VLAN;
 	default:
@@ -265,27 +333,23 @@ dispose (GObject *object)
 {
 	NMDevicePrivate *priv = NM_DEVICE_GET_PRIVATE (object);
 
-	if (priv->disposed) {
-		G_OBJECT_CLASS (nm_device_parent_class)->dispose (object);
-		return;
+	g_clear_object (&priv->proxy);
+	g_clear_object (&priv->ip4_config);
+	g_clear_object (&priv->dhcp4_config);
+	g_clear_object (&priv->ip6_config);
+	g_clear_object (&priv->dhcp6_config);
+	g_clear_object (&priv->client);
+	g_clear_object (&priv->active_connection);
+
+	if (priv->available_connections) {
+		int i;
+
+		for (i = 0; i < priv->available_connections->len; i++)
+			g_object_unref (priv->available_connections->pdata[i]);
+		g_ptr_array_free (priv->available_connections, TRUE);
+		priv->available_connections = NULL;
 	}
 
-	priv->disposed = TRUE;
-
-	g_object_unref (priv->proxy);
-	if (priv->ip4_config)
-		g_object_unref (priv->ip4_config);
-	if (priv->dhcp4_config)
-		g_object_unref (priv->dhcp4_config);
-	if (priv->ip6_config)
-		g_object_unref (priv->ip6_config);
-	if (priv->dhcp6_config)
-		g_object_unref (priv->dhcp6_config);
-	if (priv->client)
-		g_object_unref (priv->client);
-	if (priv->active_connection)
-		g_object_unref (priv->active_connection);
-
 	G_OBJECT_CLASS (nm_device_parent_class)->dispose (object);
 }
 
@@ -298,6 +362,8 @@ finalize (GObject *object)
 	g_free (priv->ip_iface);
 	g_free (priv->udi);
 	g_free (priv->driver);
+	g_free (priv->driver_version);
+	g_free (priv->firmware_version);
 	g_free (priv->product);
 	g_free (priv->vendor);
 
@@ -313,6 +379,8 @@ get_property (GObject *object,
 	NMDevice *device = NM_DEVICE (object);
 	NMDevicePrivate *priv = NM_DEVICE_GET_PRIVATE (device);
 
+	_nm_object_ensure_inited (NM_OBJECT (object));
+
 	switch (prop_id) {
 	case PROP_DEVICE_TYPE:
 		g_value_set_uint (value, nm_device_get_device_type (device));
@@ -329,12 +397,21 @@ get_property (GObject *object,
 	case PROP_DRIVER:
 		g_value_set_string (value, nm_device_get_driver (device));
 		break;
+	case PROP_DRIVER_VERSION:
+		g_value_set_string (value, nm_device_get_driver_version (device));
+		break;
+	case PROP_FIRMWARE_VERSION:
+		g_value_set_string (value, nm_device_get_firmware_version (device));
+		break;
 	case PROP_CAPABILITIES:
 		g_value_set_uint (value, nm_device_get_capabilities (device));
 		break;
 	case PROP_MANAGED:
 		g_value_set_boolean (value, nm_device_get_managed (device));
 		break;
+	case PROP_AUTOCONNECT:
+		g_value_set_boolean (value, nm_device_get_autoconnect (device));
+		break;
 	case PROP_FIRMWARE_MISSING:
 		g_value_set_boolean (value, nm_device_get_firmware_missing (device));
 		break;
@@ -364,6 +441,9 @@ get_property (GObject *object,
 	case PROP_ACTIVE_CONNECTION:
 		g_value_set_object (value, nm_device_get_active_connection (device));
 		break;
+	case PROP_AVAILABLE_CONNECTIONS:
+		g_value_set_boxed (value, nm_device_get_available_connections (device));
+		break;
 	case PROP_PRODUCT:
 		g_value_set_string (value, nm_device_get_product (device));
 		break;
@@ -384,12 +464,18 @@ set_property (GObject *object,
 {
 	NMDevice *self = NM_DEVICE (object);
 	NMDevicePrivate *priv = NM_DEVICE_GET_PRIVATE (self);
+	gboolean b;
 
 	switch (prop_id) {
 	case PROP_DEVICE_TYPE:
 		/* Construct only */
 		priv->device_type = g_value_get_uint (value);
 		break;
+	case PROP_AUTOCONNECT:
+		b = g_value_get_boolean (value);
+		if (priv->autoconnect != b)
+			nm_device_set_autoconnect (NM_DEVICE (object), b);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -482,6 +568,32 @@ nm_device_class_init (NMDeviceClass *device_class)
 						  NULL,
 						  G_PARAM_READABLE));
 
+	/**
+	 * NMDevice:driver-version:
+	 *
+	 * The version of the device driver.
+	 **/
+	g_object_class_install_property
+		(object_class, PROP_DRIVER_VERSION,
+		 g_param_spec_string (NM_DEVICE_DRIVER_VERSION,
+		                      "Driver Version",
+		                      "Driver Version",
+		                      NULL,
+		                      G_PARAM_READABLE));
+
+	/**
+	 * NMDevice:firmware-version:
+	 *
+	 * The firmware version of the device.
+	 **/
+	g_object_class_install_property
+		(object_class, PROP_FIRMWARE_VERSION,
+		 g_param_spec_string (NM_DEVICE_FIRMWARE_VERSION,
+		                      "Firmware Version",
+		                      "Firmware Version",
+		                      NULL,
+		                      G_PARAM_READABLE));
+
 	/**
 	 * NMDevice:capabilities:
 	 *
@@ -508,6 +620,19 @@ nm_device_class_init (NMDeviceClass *device_class)
 						  FALSE,
 						  G_PARAM_READABLE));
 
+	/**
+	 * NMDevice:autoconnect:
+	 *
+	 * Whether the device can auto-activate a connection.
+	 **/
+	g_object_class_install_property
+		(object_class, PROP_AUTOCONNECT,
+		 g_param_spec_boolean (NM_DEVICE_AUTOCONNECT,
+		                       "Autoconnect",
+		                       "Autoconnect",
+		                       TRUE,
+		                       G_PARAM_READWRITE));
+
 	/**
 	 * NMDevice:firmware-missing:
 	 *
@@ -613,6 +738,21 @@ nm_device_class_init (NMDeviceClass *device_class)
 		                      NM_TYPE_ACTIVE_CONNECTION,
 		                      G_PARAM_READABLE));
 
+	/**
+	 * NMDevice:available-connections:
+	 *
+	 * The available connections (#NMRemoteConnection) of the device
+	 *
+	 * Since: 0.9.8
+	 **/
+	g_object_class_install_property
+		(object_class, PROP_AVAILABLE_CONNECTIONS,
+		 g_param_spec_boxed (NM_DEVICE_AVAILABLE_CONNECTIONS,
+							 "AvailableConnections",
+							 "Available Connections",
+							 NM_TYPE_OBJECT_ARRAY,
+							 G_PARAM_READABLE));
+
 	/**
 	 * NMDevice:vendor:
 	 *
@@ -643,8 +783,10 @@ nm_device_class_init (NMDeviceClass *device_class)
 
 	/**
 	 * NMDevice::state-changed:
-	 * @device: the client that received the signal
-	 * @state: the new state of the device
+	 * @device: the device object that received the signal
+	 * @new_state: the new state of the device
+	 * @old_state: the previous state of the device
+	 * @reason: the reason describing the state change
 	 *
 	 * Notifies the state change of a #NMDevice.
 	 **/
@@ -892,6 +1034,42 @@ nm_device_get_driver (NMDevice *device)
 	return NM_DEVICE_GET_PRIVATE (device)->driver;
 }
 
+/**
+ * nm_device_get_driver_version:
+ * @device: a #NMDevice
+ *
+ * Gets the driver version of the #NMDevice.
+ *
+ * Returns: the version of the device driver. This is the internal string used by the
+ * device, and must not be modified.
+ **/
+const char *
+nm_device_get_driver_version (NMDevice *device)
+{
+	g_return_val_if_fail (NM_IS_DEVICE (device), NULL);
+
+	_nm_object_ensure_inited (NM_OBJECT (device));
+	return NM_DEVICE_GET_PRIVATE (device)->driver_version;
+}
+
+/**
+ * nm_device_get_firmware_version:
+ * @device: a #NMDevice
+ *
+ * Gets the firmware version of the #NMDevice.
+ *
+ * Returns: the firmware version of the device. This is the internal string used by the
+ * device, and must not be modified.
+ **/
+const char *
+nm_device_get_firmware_version (NMDevice *device)
+{
+	g_return_val_if_fail (NM_IS_DEVICE (device), NULL);
+
+	_nm_object_ensure_inited (NM_OBJECT (device));
+	return NM_DEVICE_GET_PRIVATE (device)->firmware_version;
+}
+
 /**
  * nm_device_get_capabilities:
  * @device: a #NMDevice
@@ -926,6 +1104,49 @@ nm_device_get_managed (NMDevice *device)
 	return NM_DEVICE_GET_PRIVATE (device)->managed;
 }
 
+/**
+ * nm_device_get_autoconnect:
+ * @device: a #NMDevice
+ *
+ * Whether the #NMDevice can be autoconnected.
+ *
+ * Returns: %TRUE if the device is allowed to be autoconnected
+ **/
+gboolean
+nm_device_get_autoconnect (NMDevice *device)
+{
+	g_return_val_if_fail (NM_IS_DEVICE (device), FALSE);
+
+	_nm_object_ensure_inited (NM_OBJECT (device));
+	return NM_DEVICE_GET_PRIVATE (device)->autoconnect;
+}
+
+/**
+ * nm_device_set_autoconnect:
+ * @device: a #NMDevice
+ * @autoconnect: %TRUE to enable autoconnecting
+ *
+ * Enables or disables automatic activation of the #NMDevice.
+ **/
+void
+nm_device_set_autoconnect (NMDevice *device, gboolean autoconnect)
+{
+	GValue value = {0,};
+
+	g_return_if_fail (NM_IS_DEVICE (device));
+
+	g_value_init (&value, G_TYPE_BOOLEAN);
+	g_value_set_boolean (&value, autoconnect);
+
+
+	NM_DEVICE_GET_PRIVATE (device)->autoconnect = autoconnect;
+
+	_nm_object_set_property (NM_OBJECT (device),
+	                         NM_DBUS_INTERFACE_DEVICE,
+	                         "Autoconnect",
+	                         &value);
+}
+
 /**
  * nm_device_get_firmware_missing:
  * @device: a #NMDevice
@@ -968,7 +1189,7 @@ nm_device_get_ip4_config (NMDevice *device)
  *
  * Gets the current #NMDHCP4Config associated with the #NMDevice.
  *
- * Returns: (transfer none): the #NMDHCPConfig or %NULL if the device is not activated or not
+ * Returns: (transfer none): the #NMDHCP4Config or %NULL if the device is not activated or not
  * using DHCP.
  **/
 NMDHCP4Config *
@@ -1003,7 +1224,7 @@ nm_device_get_ip6_config (NMDevice *device)
  *
  * Gets the current #NMDHCP6Config associated with the #NMDevice.
  *
- * Returns: (transfer none): the #NMDHCPConfig or %NULL if the device is not activated or not
+ * Returns: (transfer none): the #NMDHCP6Config or %NULL if the device is not activated or not
  * using DHCP.
  **/
 NMDHCP6Config *
@@ -1035,7 +1256,7 @@ nm_device_get_state (NMDevice *device)
 /**
  * nm_device_get_state_reason:
  * @device: a #NMDevice
- * @reason: (out) (allow-none): location to store reason (#NMDeviceStateReason), or NULL
+ * @reason: (out) (allow-none): location to store reason (#NMDeviceStateReason), or %NULL
  *
  * Gets the current #NMDevice state (return value) and the reason for entering
  * the state (@reason argument).
@@ -1071,6 +1292,28 @@ nm_device_get_active_connection (NMDevice *device)
 	return NM_DEVICE_GET_PRIVATE (device)->active_connection;
 }
 
+/**
+ * nm_device_get_available_connections:
+ * @device: a #NMDevice
+ *
+ * Gets the #NMRemoteConnections currently known to the daemon that could
+ * be activated on @device.
+ *
+ * Returns: (element-type NMClient.RemoteConnection): the #GPtrArray
+ * containing #NMRemoteConnections. This is the internal copy used by
+ * the connection, and must not be modified.
+ *
+ * Since: 0.9.8
+ **/
+const GPtrArray *
+nm_device_get_available_connections (NMDevice *device)
+{
+	g_return_val_if_fail (NM_IS_DEVICE (device), NULL);
+
+	_nm_object_ensure_inited (NM_OBJECT (device));
+	return handle_ptr_array_return (NM_DEVICE_GET_PRIVATE (device)->available_connections);
+}
+
 /* From hostap, Copyright (c) 2002-2005, Jouni Malinen <jkmaline@cc.hut.fi> */
 
 static int hex2num (char c)
@@ -1125,39 +1368,33 @@ get_decoded_property (GUdevDevice *device, const char *property)
 	return unescaped;
 }
 
-static void
-_device_update_description (NMDevice *device)
+static char *
+_get_udev_property (NMDevice *device,
+                    const char *enc_prop,  /* ID_XXX_ENC */
+                    const char *db_prop)   /* ID_XXX_FROM_DATABASE */
 {
-	NMDevicePrivate *priv;
+	NMDevicePrivate *priv = NM_DEVICE_GET_PRIVATE (device);
 	const char *subsys[3] = { "net", "tty", NULL };
 	GUdevDevice *udev_device = NULL, *tmpdev, *olddev;
 	const char *ifname;
 	guint32 count = 0;
-	const char *vendor, *model;
-
-	g_return_if_fail (NM_IS_DEVICE (device));
-	priv = NM_DEVICE_GET_PRIVATE (device);
+	char *enc_value = NULL, *db_value = NULL;
 
 	if (!priv->client) {
 		priv->client = g_udev_client_new (subsys);
 		if (!priv->client)
-			return;
+			return NULL;
 	}
 
 	ifname = nm_device_get_iface (device);
 	if (!ifname)
-		return;
+		return NULL;
 
 	udev_device = g_udev_client_query_by_subsystem_and_name (priv->client, "net", ifname);
 	if (!udev_device)
 		udev_device = g_udev_client_query_by_subsystem_and_name (priv->client, "tty", ifname);
 	if (!udev_device)
-		return;
-
-	g_free (priv->product);
-	priv->product = NULL;
-	g_free (priv->vendor);
-	priv->vendor = NULL;
+		return NULL;
 
 	/* Walk up the chain of the device and its parents a few steps to grab
 	 * vendor and device ID information off it.
@@ -1167,43 +1404,11 @@ _device_update_description (NMDevice *device)
 	 * as g_udev_device_get_parent() returns a ref-ed object.
 	 */
 	tmpdev = g_object_ref (udev_device);
-	while ((count++ < 3) && tmpdev && (!priv->vendor || !priv->product)) {
-		if (!priv->vendor)
-			priv->vendor = get_decoded_property (tmpdev, "ID_VENDOR_ENC");
-
-		if (!priv->product)
-			priv->product = get_decoded_property (tmpdev, "ID_MODEL_ENC");
-
-		olddev = tmpdev;
-		tmpdev = g_udev_device_get_parent (tmpdev);
-		g_object_unref (olddev);
-	}
-
-	/* Unref the last device if we found what we needed before running out
-	 * of parents.
-	 */
-	if (tmpdev)
-		g_object_unref (tmpdev);
-
-	/* If we didn't get strings directly from the device, try database strings */
-
-	/* Again, ref the original device as we need to unref it every iteration
-	 * since g_udev_device_get_parent() returns a refed object.
-	 */
-	tmpdev = g_object_ref (udev_device);
-	count = 0;
-	while ((count++ < 3) && tmpdev && (!priv->vendor || !priv->product)) {
-		if (!priv->vendor) {
-			vendor = g_udev_device_get_property (tmpdev, "ID_VENDOR_FROM_DATABASE");
-			if (vendor)
-				priv->vendor = g_strdup (vendor);
-		}
-
-		if (!priv->product) {
-			model = g_udev_device_get_property (tmpdev, "ID_MODEL_FROM_DATABASE");
-			if (model)
-				priv->product = g_strdup (model);
-		}
+	while ((count++ < 3) && tmpdev && !enc_value) {
+		if (!enc_value)
+			enc_value = get_decoded_property (tmpdev, enc_prop);
+		if (!db_value)
+			db_value = g_strdup (g_udev_device_get_property (tmpdev, db_prop));
 
 		olddev = tmpdev;
 		tmpdev = g_udev_device_get_parent (tmpdev);
@@ -1219,8 +1424,15 @@ _device_update_description (NMDevice *device)
 	/* Balance the initial g_udev_client_query_by_subsystem_and_name() */
 	g_object_unref (udev_device);
 
-	_nm_object_queue_notify (NM_OBJECT (device), NM_DEVICE_VENDOR);
-	_nm_object_queue_notify (NM_OBJECT (device), NM_DEVICE_PRODUCT);
+	/* Prefer the the encoded value which comes directly from the device
+	 * over the hwdata database value.
+	 */
+	if (enc_value) {
+		g_free (db_value);
+		return enc_value;
+	}
+
+	return db_value;
 }
 
 /**
@@ -1240,8 +1452,14 @@ nm_device_get_product (NMDevice *device)
 	g_return_val_if_fail (NM_IS_DEVICE (device), NULL);
 
 	priv = NM_DEVICE_GET_PRIVATE (device);
-	if (!priv->product)
-		_device_update_description (device);
+	if (!priv->product) {
+		priv->product = _get_udev_property (device, "ID_MODEL_ENC", "ID_MODEL_FROM_DATABASE");
+		if (!priv->product) {
+			/* Sometimes ID_PRODUCT_FROM_DATABASE is used? */
+			priv->product = _get_udev_property (device, "ID_MODEL_ENC", "ID_PRODUCT_FROM_DATABASE");
+		}
+		_nm_object_queue_notify (NM_OBJECT (device), NM_DEVICE_PRODUCT);
+	}
 	return priv->product;
 }
 
@@ -1262,8 +1480,10 @@ nm_device_get_vendor (NMDevice *device)
 	g_return_val_if_fail (NM_IS_DEVICE (device), NULL);
 
 	priv = NM_DEVICE_GET_PRIVATE (device);
-	if (!priv->vendor)
-		_device_update_description (device);
+	if (!priv->vendor) {
+		priv->vendor = _get_udev_property (device, "ID_VENDOR_ENC", "ID_VENDOR_FROM_DATABASE");
+		_nm_object_queue_notify (NM_OBJECT (device), NM_DEVICE_VENDOR);
+	}
 	return priv->vendor;
 }
 
