@@ -85,6 +85,16 @@ GQuark nm_setting_wireless_error_quark (void);
  */
 #define NM_SETTING_WIRELESS_MODE_ADHOC  "adhoc"
 
+/**
+ * NM_SETTING_WIRELESS_MODE_AP:
+ *
+ * Indicates AP/master mode where the wireless device is started as an access
+ * point/hotspot.
+ *
+ * Since: 0.9.8
+ */
+#define NM_SETTING_WIRELESS_MODE_AP     "ap"
+
 /**
  * NM_SETTING_WIRELESS_MODE_INFRA:
  *
