@@ -26,9 +26,11 @@
 #include <netinet/ether.h>
 
 #include <glib/gi18n.h>
+#include <gio/gio.h>
 
 #include "nm-glib-compat.h"
 #include "nm-bluez-common.h"
+#include "nm-bluez-device.h"
 #include "nm-dbus-manager.h"
 #include "nm-device-bt.h"
 #include "nm-device-private.h"
@@ -47,8 +49,8 @@
 #include "nm-enum-types.h"
 #include "nm-utils.h"
 
-#define BLUETOOTH_DUN_UUID "dun"
-#define BLUETOOTH_NAP_UUID "nap"
+#define MM_OLD_DBUS_SERVICE  "org.freedesktop.ModemManager"
+#define MM_NEW_DBUS_SERVICE  "org.freedesktop.ModemManager1"
 
 G_DEFINE_TYPE (NMDeviceBt, nm_device_bt, NM_TYPE_DEVICE)
 
@@ -57,6 +59,13 @@ G_DEFINE_TYPE (NMDeviceBt, nm_device_bt, NM_TYPE_DEVICE)
 static gboolean modem_stage1 (NMDeviceBt *self, NMModem *modem, NMDeviceStateReason *reason);
 
 typedef struct {
+	NMDBusManager *dbus_mgr;
+	guint mm_watch_id;
+	gboolean mm_running;
+
+	NMBluezDevice *bt_device;
+
+	guint8 hw_addr[ETH_ALEN];  /* binary representation of bdaddr */
 	char *bdaddr;
 	char *name;
 	guint32 capabilities;
@@ -64,9 +73,6 @@ typedef struct {
 	gboolean connected;
 	gboolean have_iface;
 
-	DBusGProxy *type_proxy;
-	DBusGProxy *dev_proxy;
-
 	char *rfcomm_iface;
 	NMModem *modem;
 	guint32 timeout_id;
@@ -79,6 +85,7 @@ enum {
 	PROP_HW_ADDRESS,
 	PROP_BT_NAME,
 	PROP_BT_CAPABILITIES,
+	PROP_BT_DEVICE,
 
 	LAST_PROP
 };
@@ -111,14 +118,6 @@ guint32 nm_device_bt_get_capabilities (NMDeviceBt *self)
 	return NM_DEVICE_BT_GET_PRIVATE (self)->capabilities;
 }
 
-const char *nm_device_bt_get_hw_address (NMDeviceBt *self)
-{
-	g_return_val_if_fail (self != NULL, NULL);
-	g_return_val_if_fail (NM_IS_DEVICE_BT (self), NULL);
-
-	return NM_DEVICE_BT_GET_PRIVATE (self)->bdaddr;
-}
-
 static guint32
 get_connection_bt_type (NMConnection *connection)
 {
@@ -141,40 +140,37 @@ get_connection_bt_type (NMConnection *connection)
 }
 
 static NMConnection *
-real_get_best_auto_connection (NMDevice *device,
-                               GSList *connections,
-                               char **specific_object)
+get_best_auto_connection (NMDevice *device,
+                          GSList *connections,
+                          char **specific_object)
 {
 	NMDeviceBtPrivate *priv = NM_DEVICE_BT_GET_PRIVATE (device);
 	GSList *iter;
 
 	for (iter = connections; iter; iter = g_slist_next (iter)) {
 		NMConnection *connection = NM_CONNECTION (iter->data);
-		NMSettingConnection *s_con;
 		guint32 bt_type;
 
-		s_con = nm_connection_get_setting_connection (connection);
-		g_assert (s_con);
-
-		if (!nm_setting_connection_get_autoconnect (s_con))
-			continue;
-
-		if (strcmp (nm_setting_connection_get_connection_type (s_con), NM_SETTING_BLUETOOTH_SETTING_NAME))
+		if (!nm_connection_is_type (connection, NM_SETTING_BLUETOOTH_SETTING_NAME))
 			continue;
 
 		bt_type = get_connection_bt_type (connection);
 		if (!(bt_type & priv->capabilities))
 			continue;
 
+		/* Can't auto-activate a DUN connection without ModemManager */
+		if (bt_type == NM_BT_CAPABILITY_DUN && priv->mm_running == FALSE)
+			continue;
+
 		return connection;
 	}
 	return NULL;
 }
 
 static gboolean
-real_check_connection_compatible (NMDevice *device,
-                                  NMConnection *connection,
-                                  GError **error)
+check_connection_compatible (NMDevice *device,
+                             NMConnection *connection,
+                             GError **error)
 {
 	NMDeviceBtPrivate *priv = NM_DEVICE_BT_GET_PRIVATE (device);
 	NMSettingConnection *s_con;
@@ -228,11 +224,28 @@ real_check_connection_compatible (NMDevice *device,
 }
 
 static gboolean
-real_complete_connection (NMDevice *device,
-                          NMConnection *connection,
-                          const char *specific_object,
-                          const GSList *existing_connections,
-                          GError **error)
+check_connection_available (NMDevice *device, NMConnection *connection)
+{
+	NMDeviceBtPrivate *priv = NM_DEVICE_BT_GET_PRIVATE (device);
+	guint32 bt_type;
+
+	bt_type = get_connection_bt_type (connection);
+	if (!(bt_type & priv->capabilities))
+		return FALSE;
+
+	/* DUN connections aren't available without ModemManager */
+	if (bt_type == NM_BT_CAPABILITY_DUN && priv->mm_running == FALSE)
+		return FALSE;
+
+	return TRUE;
+}
+
+static gboolean
+complete_connection (NMDevice *device,
+                     NMConnection *connection,
+                     const char *specific_object,
+                     const GSList *existing_connections,
+                     GError **error)
 {
 	NMDeviceBtPrivate *priv = NM_DEVICE_BT_GET_PRIVATE (device);
 	NMSettingBluetooth *s_bt;
@@ -369,11 +382,20 @@ real_complete_connection (NMDevice *device,
 }
 
 static guint32
-real_get_generic_capabilities (NMDevice *dev)
+get_generic_capabilities (NMDevice *dev)
 {
 	return NM_DEVICE_CAP_NM_SUPPORTED;
 }
 
+static const guint8 *
+get_hw_address (NMDevice *device, guint *out_len)
+{
+	NMDeviceBtPrivate *priv = NM_DEVICE_BT_GET_PRIVATE (device);
+
+	*out_len = sizeof (priv->hw_addr);
+	return priv->hw_addr;
+}
+
 static gboolean
 hwaddr_matches (NMDevice *device,
                 NMConnection *connection,
@@ -517,8 +539,7 @@ static void
 device_state_changed (NMDevice *device,
                       NMDeviceState new_state,
                       NMDeviceState old_state,
-                      NMDeviceStateReason reason,
-                      gpointer user_data)
+                      NMDeviceStateReason reason)
 {
 	NMDeviceBtPrivate *priv = NM_DEVICE_BT_GET_PRIVATE (device);
 
@@ -528,7 +549,6 @@ device_state_changed (NMDevice *device,
 
 static void
 modem_ip4_config_result (NMModem *self,
-                         const char *iface,
                          NMIP4Config *config,
                          GError *error,
                          gpointer user_data)
@@ -545,12 +565,16 @@ modem_ip4_config_result (NMModem *self,
 		             error && error->message ? error->message : "(unknown)");
 
 		nm_device_state_changed (device, NM_DEVICE_STATE_FAILED, NM_DEVICE_STATE_REASON_IP_CONFIG_UNAVAILABLE);
-	} else {
-		if (iface)
-			nm_device_set_ip_iface (device, iface);
-
+	} else
 		nm_device_activate_schedule_ip4_config_result (device, config);
-	}
+}
+
+static void
+data_port_changed_cb (NMModem *modem, GParamSpec *pspec, gpointer user_data)
+{
+	NMDevice *self = NM_DEVICE (user_data);
+
+	nm_device_set_ip_iface (self, nm_modem_get_data_port (modem));
 }
 
 static gboolean
@@ -586,7 +610,8 @@ nm_device_bt_modem_added (NMDeviceBt *self,
                           const char *driver)
 {
 	NMDeviceBtPrivate *priv;
-	const char *modem_iface;
+	const gchar *modem_data_port;
+	const gchar *modem_control_port;
 	char *base;
 	NMDeviceState state;
 	NMDeviceStateReason reason = NM_DEVICE_STATE_REASON_NONE;
@@ -597,14 +622,15 @@ nm_device_bt_modem_added (NMDeviceBt *self,
 	g_return_val_if_fail (NM_IS_MODEM (modem), FALSE);
 
 	priv = NM_DEVICE_BT_GET_PRIVATE (self);
-	modem_iface = nm_modem_get_iface (modem);
-	g_return_val_if_fail (modem_iface != NULL, FALSE);
+	modem_data_port = nm_modem_get_data_port (modem);
+	modem_control_port = nm_modem_get_control_port (modem);
+	g_return_val_if_fail (modem_data_port != NULL || modem_control_port != NULL, FALSE);
 
 	if (!priv->rfcomm_iface)
 		return FALSE;
 
 	base = g_path_get_basename (priv->rfcomm_iface);
-	if (strcmp (base, modem_iface)) {
+	if (g_strcmp0 (base, modem_data_port) && g_strcmp0 (base, modem_control_port)) {
 		g_free (base);
 		return FALSE;
 	}
@@ -645,6 +671,13 @@ nm_device_bt_modem_added (NMDeviceBt *self,
 	g_signal_connect (modem, NM_MODEM_AUTH_REQUESTED, G_CALLBACK (modem_auth_requested), self);
 	g_signal_connect (modem, NM_MODEM_AUTH_RESULT, G_CALLBACK (modem_auth_result), self);
 
+	/* In the old ModemManager the data port is known from the very beginning;
+	 * while in the new ModemManager the data port is set afterwards when the bearer gets
+	 * created */
+	if (modem_data_port)
+		nm_device_set_ip_iface (NM_DEVICE (self), modem_data_port);
+	g_signal_connect (modem, "notify::" NM_MODEM_DATA_PORT, G_CALLBACK (data_port_changed_cb), self);
+
 	/* Kick off the modem connection */
 	if (!modem_stage1 (self, modem, &reason))
 		nm_device_state_changed (NM_DEVICE (self), NM_DEVICE_STATE_FAILED, reason);
@@ -668,14 +701,18 @@ nm_device_bt_modem_removed (NMDeviceBt *self, NMModem *modem)
 	if (modem != priv->modem)
 		return FALSE;
 
+	/* Fail the device if the modem was removed while active */
 	state = nm_device_get_state (NM_DEVICE (self));
-	nm_modem_device_state_changed (priv->modem,
-	                               NM_DEVICE_STATE_DISCONNECTED,
-	                               state,
-	                               NM_DEVICE_STATE_REASON_USER_REQUESTED);
+	if (   state == NM_DEVICE_STATE_ACTIVATED
+	    || nm_device_is_activating (NM_DEVICE (self))) {
+		nm_device_state_changed (NM_DEVICE (self),
+		                         NM_DEVICE_STATE_FAILED,
+		                         NM_DEVICE_STATE_REASON_BT_FAILED);
+	} else {
+		g_object_unref (priv->modem);
+		priv->modem = NULL;
+	}
 
-	g_object_unref (priv->modem);
-	priv->modem = NULL;
 	return TRUE;
 }
 
@@ -729,18 +766,19 @@ check_connect_continue (NMDeviceBt *self)
 }
 
 static void
-bluez_connect_cb (DBusGProxy *proxy,
-                  DBusGProxyCall *call_id,
+bluez_connect_cb (GObject *object,
+                  GAsyncResult *res,
                   void *user_data)
 {
 	NMDeviceBt *self = NM_DEVICE_BT (user_data);
 	NMDeviceBtPrivate *priv = NM_DEVICE_BT_GET_PRIVATE (self);
 	GError *error = NULL;
-	char *device;
+	const char *device;
 
-	if (dbus_g_proxy_end_call (proxy, call_id, &error,
-	                           G_TYPE_STRING, &device,
-	                           G_TYPE_INVALID) == FALSE) {
+	device = nm_bluez_device_connect_finish (NM_BLUEZ_DEVICE (object),
+	                                         res, &error);
+
+	if (!device) {
 		nm_log_warn (LOGD_BT, "Error connecting with bluez: %s",
 		             error && error->message ? error->message : "(unknown)");
 		g_clear_error (&error);
@@ -751,20 +789,11 @@ bluez_connect_cb (DBusGProxy *proxy,
 		return;
 	}
 
-	if (!device || !strlen (device)) {
-		nm_log_warn (LOGD_BT, "Invalid network device returned by bluez");
-
-		nm_device_state_changed (NM_DEVICE (self),
-		                         NM_DEVICE_STATE_FAILED,
-		                         NM_DEVICE_STATE_REASON_BT_FAILED);
-	}
-
 	if (priv->bt_type == NM_BT_CAPABILITY_DUN) {
 		g_free (priv->rfcomm_iface);
-		priv->rfcomm_iface = device;
+		priv->rfcomm_iface = g_strdup (device);
 	} else if (priv->bt_type == NM_BT_CAPABILITY_NAP) {
 		nm_device_set_ip_iface (NM_DEVICE (self), device);
-		g_free (device);
 	}
 
 	nm_log_dbg (LOGD_BT, "(%s): connect request successful",
@@ -776,33 +805,17 @@ bluez_connect_cb (DBusGProxy *proxy,
 }
 
 static void
-bluez_property_changed (DBusGProxy *proxy,
-                        const char *property,
-                        GValue *value,
-                        gpointer user_data)
+bluez_connected_changed (NMBluezDevice *bt_device,
+                         GParamSpec *pspec,
+                         NMDevice *device)
 {
-	NMDevice *device = NM_DEVICE (user_data);
-	NMDeviceBt *self = NM_DEVICE_BT (user_data);
+	NMDeviceBt *self = NM_DEVICE_BT (device);
 	NMDeviceBtPrivate *priv = NM_DEVICE_BT_GET_PRIVATE (self);
 	gboolean connected;
 	NMDeviceState state;
-	const char *prop_str = "(unknown)";
-
-	if (G_VALUE_HOLDS_STRING (value))
-		prop_str = g_value_get_string (value);
-	else if (G_VALUE_HOLDS_BOOLEAN (value))
-		prop_str = g_value_get_boolean (value) ? "true" : "false";
-
-	nm_log_dbg (LOGD_BT, "(%s): bluez property '%s' changed to '%s'",
-	            nm_device_get_iface (device),
-	            property,
-	            prop_str);
-
-	if (strcmp (property, "Connected"))
-		return;
 
 	state = nm_device_get_state (device);
-	connected = g_value_get_boolean (value);
+	connected = nm_bluez_device_get_connected (bt_device);
 	if (connected) {
 		if (state == NM_DEVICE_STATE_CONFIG) {
 			nm_log_dbg (LOGD_BT, "(%s): connected to the device",
@@ -850,12 +863,9 @@ bt_connect_timeout (gpointer user_data)
 }
 
 static NMActStageReturn
-real_act_stage2_config (NMDevice *device, NMDeviceStateReason *reason)
+act_stage2_config (NMDevice *device, NMDeviceStateReason *reason)
 {
 	NMDeviceBtPrivate *priv = NM_DEVICE_BT_GET_PRIVATE (device);
-	NMDBusManager *dbus_mgr;
-	DBusGConnection *g_connection;
-	gboolean dun = FALSE;
 	NMConnection *connection;
 
 	connection = nm_device_get_connection (device);
@@ -866,42 +876,8 @@ real_act_stage2_config (NMDevice *device, NMDeviceStateReason *reason)
 		return NM_ACT_STAGE_RETURN_FAILURE;
 	}
 
-	dbus_mgr = nm_dbus_manager_get ();
-	g_connection = nm_dbus_manager_get_connection (dbus_mgr);
-	g_object_unref (dbus_mgr);
-
-	if (priv->bt_type == NM_BT_CAPABILITY_DUN)
-		dun = TRUE;
-	else if (priv->bt_type == NM_BT_CAPABILITY_NAP)
-		dun = FALSE;
-	else
-		g_assert_not_reached ();
-
-	priv->dev_proxy = dbus_g_proxy_new_for_name (g_connection,
-	                                               BLUEZ_SERVICE,
-	                                               nm_device_get_udi (device),
-	                                               BLUEZ_DEVICE_INTERFACE);
-	if (!priv->dev_proxy) {
-		// FIXME: set a reason code
-		return NM_ACT_STAGE_RETURN_FAILURE;
-	}
-
-	/* Watch for BT device property changes */
-	dbus_g_object_register_marshaller (_nm_marshal_VOID__STRING_BOXED,
-	                                   G_TYPE_NONE,
-	                                   G_TYPE_STRING, G_TYPE_VALUE,
-	                                   G_TYPE_INVALID);
-	dbus_g_proxy_add_signal (priv->dev_proxy, "PropertyChanged",
-	                         G_TYPE_STRING, G_TYPE_VALUE, G_TYPE_INVALID);
-	dbus_g_proxy_connect_signal (priv->dev_proxy, "PropertyChanged",
-	                             G_CALLBACK (bluez_property_changed), device, NULL);
-
-	priv->type_proxy = dbus_g_proxy_new_for_name (g_connection,
-	                                              BLUEZ_SERVICE,
-	                                              nm_device_get_udi (device),
-	                                              dun ? BLUEZ_SERIAL_INTERFACE : BLUEZ_NETWORK_INTERFACE);
-	if (!priv->type_proxy) {
-		// FIXME: set a reason code
+	if (priv->bt_type == NM_BT_CAPABILITY_DUN && !priv->mm_running) {
+		*reason = NM_DEVICE_STATE_REASON_MODEM_MANAGER_UNAVAILABLE;
 		return NM_ACT_STAGE_RETURN_FAILURE;
 	}
 
@@ -909,13 +885,9 @@ real_act_stage2_config (NMDevice *device, NMDeviceStateReason *reason)
 	            nm_device_get_iface (device));
 
 	/* Connect to the BT device */
-	dbus_g_proxy_begin_call_with_timeout (priv->type_proxy, "Connect",
-	                                      bluez_connect_cb,
-	                                      device,
-	                                      NULL,
-	                                      20000,
-	                                      G_TYPE_STRING, dun ? BLUETOOTH_DUN_UUID : BLUETOOTH_NAP_UUID,
-	                                      G_TYPE_INVALID);
+	nm_bluez_device_connect_async (priv->bt_device,
+	                               priv->bt_type & (NM_BT_CAPABILITY_DUN | NM_BT_CAPABILITY_NAP),
+	                               bluez_connect_cb, device);
 
 	if (priv->timeout_id)
 		g_source_remove (priv->timeout_id);
@@ -925,9 +897,9 @@ real_act_stage2_config (NMDevice *device, NMDeviceStateReason *reason)
 }
 
 static NMActStageReturn
-real_act_stage3_ip4_config_start (NMDevice *device,
-                                  NMIP4Config **out_config,
-                                  NMDeviceStateReason *reason)
+act_stage3_ip4_config_start (NMDevice *device,
+                             NMIP4Config **out_config,
+                             NMDeviceStateReason *reason)
 {
 	NMDeviceBtPrivate *priv = NM_DEVICE_BT_GET_PRIVATE (device);
 	NMActStageReturn ret;
@@ -944,9 +916,9 @@ real_act_stage3_ip4_config_start (NMDevice *device,
 }
 
 static NMActStageReturn
-real_act_stage3_ip6_config_start (NMDevice *device,
-                                  NMIP6Config **out_config,
-                                  NMDeviceStateReason *reason)
+act_stage3_ip6_config_start (NMDevice *device,
+                             NMIP6Config **out_config,
+                             NMDeviceStateReason *reason)
 {
 	NMDeviceBtPrivate *priv = NM_DEVICE_BT_GET_PRIVATE (device);
 	NMActStageReturn ret;
@@ -963,7 +935,7 @@ real_act_stage3_ip6_config_start (NMDevice *device,
 }
 
 static void
-real_deactivate (NMDevice *device)
+deactivate (NMDevice *device)
 {
 	NMDeviceBtPrivate *priv = NM_DEVICE_BT_GET_PRIVATE (device);
 
@@ -971,7 +943,6 @@ real_deactivate (NMDevice *device)
 	priv->connected = FALSE;
 
 	if (priv->bt_type == NM_BT_CAPABILITY_DUN) {
-
 		if (priv->modem) {
 			nm_modem_deactivate (priv->modem, device);
 
@@ -985,32 +956,10 @@ real_deactivate (NMDevice *device)
 			g_object_unref (priv->modem);
 			priv->modem = NULL;
 		}
-
-		if (priv->type_proxy) {
-			/* Don't ever pass NULL through dbus; rfcomm_iface
-			 * might happen to be NULL for some reason.
-			 */
-			if (priv->rfcomm_iface) {
-				dbus_g_proxy_call_no_reply (priv->type_proxy, "Disconnect",
-				                            G_TYPE_STRING, priv->rfcomm_iface,
-				                            G_TYPE_INVALID);
-			}
-			g_object_unref (priv->type_proxy);
-			priv->type_proxy = NULL;
-		}
-	} else if (priv->bt_type == NM_BT_CAPABILITY_NAP) {
-		if (priv->type_proxy) {
-			dbus_g_proxy_call_no_reply (priv->type_proxy, "Disconnect",
-			                            G_TYPE_INVALID);
-			g_object_unref (priv->type_proxy);
-			priv->type_proxy = NULL;
-		}
 	}
 
-	if (priv->dev_proxy) {
-		g_object_unref (priv->dev_proxy);
-		priv->dev_proxy = NULL;
-	}
+	if (priv->bt_type != NM_BT_CAPABILITY_NONE)
+		nm_bluez_device_disconnect (priv->bt_device);
 
 	if (priv->timeout_id) {
 		g_source_remove (priv->timeout_id);
@@ -1028,40 +977,156 @@ real_deactivate (NMDevice *device)
 
 /*****************************************************************************/
 
+static gboolean
+is_available (NMDevice *dev)
+{
+	NMDeviceBt *self = NM_DEVICE_BT (dev);
+	NMDeviceBtPrivate *priv = NM_DEVICE_BT_GET_PRIVATE (self);
+
+	/* PAN doesn't need ModemManager, so devices that support it are always available */
+	if (priv->capabilities & NM_BT_CAPABILITY_NAP)
+		return TRUE;
+
+	/* DUN requires ModemManager */
+	return priv->mm_running;
+}
+
+static void
+handle_availability_change (NMDeviceBt *self,
+                            gboolean old_available,
+                            NMDeviceStateReason unavailable_reason)
+{
+	NMDevice *device = NM_DEVICE (self);
+	NMDeviceState state;
+	gboolean available;
+
+	state = nm_device_get_state (device);
+	if (state < NM_DEVICE_STATE_UNAVAILABLE) {
+		nm_log_dbg (LOGD_BT, "(%s): availability blocked by UNMANAGED state",
+		            nm_device_get_iface (device));
+		return;
+	}
+
+	available = nm_device_is_available (device);
+	if (available == old_available)
+		return;
+
+	if (available) {
+		if (state != NM_DEVICE_STATE_UNAVAILABLE)
+			nm_log_warn (LOGD_CORE | LOGD_BT, "not in expected unavailable state!");
+
+		nm_device_state_changed (device,
+		                         NM_DEVICE_STATE_DISCONNECTED,
+		                         NM_DEVICE_STATE_REASON_NONE);
+	} else {
+		nm_device_state_changed (device,
+		                         NM_DEVICE_STATE_UNAVAILABLE,
+		                         unavailable_reason);
+	}
+}
+
+static void
+set_mm_running (NMDeviceBt *self, gboolean running)
+{
+	NMDeviceBtPrivate *priv = NM_DEVICE_BT_GET_PRIVATE (self);
+	gboolean old_available;
+
+	if (priv->mm_running == running)
+		return;
+
+	nm_log_dbg (LOGD_BT, "(%s): ModemManager now %s",
+	            nm_device_get_iface (NM_DEVICE (self)),
+	            running ? "available" : "unavailable");
+
+	old_available = nm_device_is_available (NM_DEVICE (self));
+	priv->mm_running = running;
+	handle_availability_change (self, old_available, NM_DEVICE_STATE_REASON_MODEM_MANAGER_UNAVAILABLE);
+
+	/* Need to recheck available connections whenever MM appears or disappears,
+	 * since the device could be both DUN and NAP capable and thus may not
+	 * change state (which rechecks available connections) when MM comes and goes.
+	 */
+	if (priv->capabilities & NM_BT_CAPABILITY_DUN)
+	    nm_device_recheck_available_connections (NM_DEVICE (self));
+}
+
+static void
+mm_name_owner_changed (NMDBusManager *dbus_mgr,
+                       const char *name,
+                       const char *old_owner,
+                       const char *new_owner,
+                       NMDeviceBt *self)
+{
+	gboolean old_owner_good;
+	gboolean new_owner_good;
+
+	/* Can't handle the signal if its not from the modem service */
+	if (   strcmp (MM_OLD_DBUS_SERVICE, name) != 0
+#if WITH_MODEM_MANAGER_1
+	    && strcmp (MM_NEW_DBUS_SERVICE, name) != 0
+#endif
+	    )
+		return;
+
+	old_owner_good = (old_owner && strlen (old_owner));
+	new_owner_good = (new_owner && strlen (new_owner));
+
+	if (!old_owner_good && new_owner_good)
+		set_mm_running (self, TRUE);
+	else if (old_owner_good && !new_owner_good)
+		set_mm_running (self, FALSE);
+}
+
+/*****************************************************************************/
+
 NMDevice *
-nm_device_bt_new (const char *udi,
+nm_device_bt_new (NMBluezDevice *bt_device,
+                  const char *udi,
                   const char *bdaddr,
                   const char *name,
                   guint32 capabilities,
                   gboolean managed)
 {
-	NMDevice *device;
-
 	g_return_val_if_fail (udi != NULL, NULL);
 	g_return_val_if_fail (bdaddr != NULL, NULL);
 	g_return_val_if_fail (name != NULL, NULL);
 	g_return_val_if_fail (capabilities != NM_BT_CAPABILITY_NONE, NULL);
+	g_return_val_if_fail (NM_IS_BLUEZ_DEVICE (bt_device), NULL);
 
-	device = (NMDevice *) g_object_new (NM_TYPE_DEVICE_BT,
-	                                    NM_DEVICE_UDI, udi,
-	                                    NM_DEVICE_IFACE, bdaddr,
-	                                    NM_DEVICE_DRIVER, "bluez",
-	                                    NM_DEVICE_BT_HW_ADDRESS, bdaddr,
-	                                    NM_DEVICE_BT_NAME, name,
-	                                    NM_DEVICE_BT_CAPABILITIES, capabilities,
-	                                    NM_DEVICE_MANAGED, managed,
-	                                    NM_DEVICE_TYPE_DESC, "Bluetooth",
-	                                    NM_DEVICE_DEVICE_TYPE, NM_DEVICE_TYPE_BT,
-	                                    NULL);
-	if (device)
-		g_signal_connect (device, "state-changed", G_CALLBACK (device_state_changed), device);
-
-	return device;
+	return (NMDevice *) g_object_new (NM_TYPE_DEVICE_BT,
+	                                  NM_DEVICE_UDI, udi,
+	                                  NM_DEVICE_IFACE, bdaddr,
+	                                  NM_DEVICE_DRIVER, "bluez",
+	                                  NM_DEVICE_BT_HW_ADDRESS, bdaddr,
+	                                  NM_DEVICE_BT_DEVICE, bt_device,
+	                                  NM_DEVICE_BT_NAME, name,
+	                                  NM_DEVICE_BT_CAPABILITIES, capabilities,
+	                                  NM_DEVICE_MANAGED, managed,
+	                                  NM_DEVICE_TYPE_DESC, "Bluetooth",
+	                                  NM_DEVICE_DEVICE_TYPE, NM_DEVICE_TYPE_BT,
+	                                  NULL);
 }
 
 static void
 nm_device_bt_init (NMDeviceBt *self)
 {
+	NMDeviceBtPrivate *priv = NM_DEVICE_BT_GET_PRIVATE (self);
+	gboolean mm_running;
+
+	priv->dbus_mgr = nm_dbus_manager_get ();
+
+	priv->mm_watch_id = g_signal_connect (priv->dbus_mgr,
+	                                      NM_DBUS_MANAGER_NAME_OWNER_CHANGED,
+	                                      G_CALLBACK (mm_name_owner_changed),
+	                                      self);
+
+	/* Initial check to see if ModemManager is running */
+	mm_running = nm_dbus_manager_name_has_owner (priv->dbus_mgr, MM_OLD_DBUS_SERVICE);
+#if WITH_MODEM_MANAGER_1
+	if (!mm_running)
+		mm_running = nm_dbus_manager_name_has_owner (priv->dbus_mgr, MM_NEW_DBUS_SERVICE);
+#endif
+	set_mm_running (self, mm_running);
 }
 
 static void
@@ -1074,6 +1139,8 @@ set_property (GObject *object, guint prop_id,
 	case PROP_HW_ADDRESS:
 		/* Construct only */
 		priv->bdaddr = g_ascii_strup (g_value_get_string (value), -1);
+		if (!nm_utils_hwaddr_aton (priv->bdaddr, ARPHRD_ETHER, &priv->hw_addr))
+			nm_log_err (LOGD_HW, "Failed to convert BT address '%s'", priv->bdaddr);
 		break;
 	case PROP_BT_NAME:
 		/* Construct only */
@@ -1083,6 +1150,15 @@ set_property (GObject *object, guint prop_id,
 		/* Construct only */
 		priv->capabilities = g_value_get_uint (value);
 		break;
+	case PROP_BT_DEVICE:
+		/* Construct only */
+		priv->bt_device = g_value_dup_object (value);
+
+		/* Watch for BT device property changes */
+		g_signal_connect (priv->bt_device, "notify::" NM_BLUEZ_DEVICE_CONNECTED,
+		                  G_CALLBACK (bluez_connected_changed),
+		                  object);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -1105,6 +1181,9 @@ get_property (GObject *object, guint prop_id,
 	case PROP_BT_CAPABILITIES:
 		g_value_set_uint (value, priv->capabilities);
 		break;
+	case PROP_BT_DEVICE:
+		g_value_set_object (value, priv->bt_device);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -1112,7 +1191,7 @@ get_property (GObject *object, guint prop_id,
 }
 
 static void
-finalize (GObject *object)
+dispose (GObject *object)
 {
 	NMDeviceBtPrivate *priv = NM_DEVICE_BT_GET_PRIVATE (object);
 
@@ -1121,14 +1200,26 @@ finalize (GObject *object)
 		priv->timeout_id = 0;
 	}
 
-	if (priv->type_proxy)
-		g_object_unref (priv->type_proxy);
+	g_signal_handlers_disconnect_by_func (priv->bt_device,
+	                                      G_CALLBACK (bluez_connected_changed),
+	                                      object);
 
-	if (priv->dev_proxy)
-		g_object_unref (priv->dev_proxy);
+	if (priv->dbus_mgr && priv->mm_watch_id) {
+		g_signal_handler_disconnect (priv->dbus_mgr, priv->mm_watch_id);
+		priv->mm_watch_id = 0;
+	}
+	g_clear_object (&priv->dbus_mgr);
 
-	if (priv->modem)
-		g_object_unref (priv->modem);
+	g_clear_object (&priv->modem);
+	g_clear_object (&priv->bt_device);
+
+	G_OBJECT_CLASS (nm_device_bt_parent_class)->dispose (object);
+}
+
+static void
+finalize (GObject *object)
+{
+	NMDeviceBtPrivate *priv = NM_DEVICE_BT_GET_PRIVATE (object);
 
 	g_free (priv->rfcomm_iface);
 	g_free (priv->bdaddr);
@@ -1147,17 +1238,23 @@ nm_device_bt_class_init (NMDeviceBtClass *klass)
 
 	object_class->get_property = get_property;
 	object_class->set_property = set_property;
+	object_class->dispose = dispose;
 	object_class->finalize = finalize;
 
-	device_class->get_best_auto_connection = real_get_best_auto_connection;
-	device_class->get_generic_capabilities = real_get_generic_capabilities;
-	device_class->deactivate = real_deactivate;
-	device_class->act_stage2_config = real_act_stage2_config;
-	device_class->act_stage3_ip4_config_start = real_act_stage3_ip4_config_start;
-	device_class->act_stage3_ip6_config_start = real_act_stage3_ip6_config_start;
-	device_class->check_connection_compatible = real_check_connection_compatible;
-	device_class->complete_connection = real_complete_connection;
+	device_class->get_best_auto_connection = get_best_auto_connection;
+	device_class->get_generic_capabilities = get_generic_capabilities;
+	device_class->deactivate = deactivate;
+	device_class->act_stage2_config = act_stage2_config;
+	device_class->act_stage3_ip4_config_start = act_stage3_ip4_config_start;
+	device_class->act_stage3_ip6_config_start = act_stage3_ip6_config_start;
+	device_class->check_connection_compatible = check_connection_compatible;
+	device_class->check_connection_available = check_connection_available;
+	device_class->complete_connection = complete_connection;
 	device_class->hwaddr_matches = hwaddr_matches;
+	device_class->get_hw_address = get_hw_address;
+	device_class->is_available = is_available;
+
+	device_class->state_changed = device_state_changed;
 
 	/* Properties */
 	g_object_class_install_property
@@ -1184,6 +1281,14 @@ nm_device_bt_class_init (NMDeviceBtClass *klass)
 		                    NM_BT_CAPABILITY_NONE, G_MAXUINT, NM_BT_CAPABILITY_NONE,
 		                    G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+	g_object_class_install_property
+		(object_class, PROP_BT_DEVICE,
+		 g_param_spec_object (NM_DEVICE_BT_DEVICE,
+		                      "NMBluezDevice object for the Device",
+		                      "NMBluezDevice object for the Device",
+		                      NM_TYPE_BLUEZ_DEVICE,
+		                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | NM_PROPERTY_PARAM_NO_EXPORT));
+
 	/* Signals */
 	signals[PPP_STATS] =
 		g_signal_new ("ppp-stats",
@@ -1195,7 +1300,7 @@ nm_device_bt_class_init (NMDeviceBtClass *klass)
 		              G_TYPE_NONE, 2,
 		              G_TYPE_UINT, G_TYPE_UINT);
 
-	signals[PROPERTIES_CHANGED] = 
+	signals[PROPERTIES_CHANGED] =
 		nm_properties_changed_signal_new (object_class,
 		                                  G_STRUCT_OFFSET (NMDeviceBtClass, properties_changed));
 
