@@ -27,7 +27,6 @@
 
 #include <unistd.h>
 #include <string.h>
-#include <ctype.h>
 #include <gmodule.h>
 #include <net/if_arp.h>
 #include <pwd.h>
@@ -287,7 +286,7 @@ connection_sort (gconstpointer pa, gconstpointer pb)
 	NMSettingConnection *con_a;
 	NMConnection *b = NM_CONNECTION (pb);
 	NMSettingConnection *con_b;
-	guint64 ts_a, ts_b;
+	guint64 ts_a = 0, ts_b = 0;
 
 	con_a = nm_connection_get_setting_connection (a);
 	g_assert (con_a);
@@ -300,8 +299,8 @@ connection_sort (gconstpointer pa, gconstpointer pb)
 		return 1;
 	}
 
-	ts_a = nm_settings_connection_get_timestamp (NM_SETTINGS_CONNECTION (pa));
-	ts_b = nm_settings_connection_get_timestamp (NM_SETTINGS_CONNECTION (pb));
+	nm_settings_connection_get_timestamp (NM_SETTINGS_CONNECTION (pa), &ts_a);
+	nm_settings_connection_get_timestamp (NM_SETTINGS_CONNECTION (pb), &ts_b);
 	if (ts_a > ts_b)
 		return -1;
 	else if (ts_a == ts_b)
@@ -588,7 +587,7 @@ load_plugins (NMSettings *self, const char **plugins, GError **error)
 	gboolean keyfile_added = FALSE;
 	gboolean success = TRUE;
 
-	for (iter = plugins; *iter; iter++) {
+	for (iter = plugins; iter && *iter; iter++) {
 		GModule *plugin;
 		char *full_name, *path;
 		const char *pname = *iter;
@@ -596,7 +595,7 @@ load_plugins (NMSettings *self, const char **plugins, GError **error)
 		GObject * (*factory_func) (const char *);
 
 		/* strip leading spaces */
-		while (isblank (*pname))
+		while (g_ascii_isspace (*pname))
 			pname++;
 
 		/* ifcfg-fedora was renamed ifcfg-rh; handle old configs here */
@@ -948,6 +947,15 @@ add_new_connection (NMSettings *self,
 	return NULL;
 }
 
+static NMConnection *
+_nm_connection_provider_add_connection (NMConnectionProvider *provider,
+                                        NMConnection *connection,
+                                        GError **error)
+{
+	g_assert (NM_IS_CONNECTION_PROVIDER (provider) && NM_IS_SETTINGS (provider));
+	return (NMConnection *) add_new_connection (NM_SETTINGS (provider), connection, error);
+}
+
 static gboolean
 secrets_filter_cb (NMSetting *setting,
                    const char *secret,
@@ -1163,18 +1171,14 @@ nm_settings_add_connection (NMSettings *self,
 	/* Ensure the caller's username exists in the connection's permissions,
 	 * or that the permissions is empty (ie, visible by everyone).
 	 */
-	if (0 != caller_uid) {
-		if (!nm_auth_uid_in_acl (connection, priv->session_monitor, caller_uid, &error_desc)) {
-			error = g_error_new_literal (NM_SETTINGS_ERROR,
-			                             NM_SETTINGS_ERROR_NOT_PRIVILEGED,
-			                             error_desc);
-			g_free (error_desc);
-			callback (self, NULL, error, context, user_data);
-			g_error_free (error);
-			return;
-		}
-
-		/* Caller is allowed to add this connection */
+	if (!nm_auth_uid_in_acl (connection, priv->session_monitor, caller_uid, &error_desc)) {
+		error = g_error_new_literal (NM_SETTINGS_ERROR,
+		                             NM_SETTINGS_ERROR_PERMISSION_DENIED,
+		                             error_desc);
+		g_free (error_desc);
+		callback (self, NULL, error, context, user_data);
+		g_error_free (error);
+		return;
 	}
 
 	/* If the caller is the only user in the connection's permissions, then
@@ -1380,12 +1384,13 @@ have_connection_for_device (NMSettings *self, GByteArray *mac, NMDevice *device)
 
 /* Search through the list of blacklisted MAC addresses in the config file. */
 static gboolean
-is_mac_auto_wired_blacklisted (NMSettings *self, const GByteArray *mac, int hwaddr_type)
+is_mac_auto_wired_blacklisted (NMSettings *self, const GByteArray *mac)
 {
 	NMSettingsPrivate *priv = NM_SETTINGS_GET_PRIVATE (self);
 	GKeyFile *config;
 	char **list, **iter;
 	gboolean found = FALSE;
+	int hwaddr_type;
 
 	g_return_val_if_fail (mac != NULL, FALSE);
 
@@ -1402,6 +1407,8 @@ is_mac_auto_wired_blacklisted (NMSettings *self, const GByteArray *mac, int hwad
 	if (!g_key_file_load_from_file (config, priv->config_file, G_KEY_FILE_NONE, NULL))
 		goto out;
 
+	hwaddr_type = nm_utils_hwaddr_type (mac->len);
+
 	list = g_key_file_get_string_list (config, "main", CONFIG_KEY_NO_AUTO_DEFAULT, NULL, NULL);
 	for (iter = list; iter && *iter; iter++) {
 		guint8 *candidate, buffer[NM_UTILS_HWADDR_LEN_MAX];
@@ -1585,7 +1592,7 @@ nm_settings_device_added (NMSettings *self, NMDevice *device)
 	NMSettingsPrivate *priv = NM_SETTINGS_GET_PRIVATE (self);
 	GByteArray *mac = NULL;
 	const guint8 *hwaddr;
-	int hwaddr_type;
+	guint hwaddr_len = 0;
 	NMDefaultWiredConnection *wired;
 	gboolean read_only = TRUE;
 	const char *id;
@@ -1601,14 +1608,13 @@ nm_settings_device_added (NMSettings *self, NMDevice *device)
 	    || g_object_get_data (G_OBJECT (device), DEFAULT_WIRED_TAG))
 		return;
 
-	hwaddr = nm_device_wired_get_hwaddr (NM_DEVICE_WIRED (device));
-	hwaddr_type = nm_device_wired_get_hwaddr_type (NM_DEVICE_WIRED (device));
+	hwaddr = nm_device_get_hw_address (device, &hwaddr_len);
 
-	mac = g_byte_array_new ();
-	g_byte_array_append (mac, hwaddr, nm_utils_hwaddr_len (hwaddr_type));
+	mac = g_byte_array_sized_new (hwaddr_len);
+	g_byte_array_append (mac, hwaddr, hwaddr_len);
 
 	if (   have_connection_for_device (self, mac, device)
-		|| is_mac_auto_wired_blacklisted (self, mac, hwaddr_type))
+		|| is_mac_auto_wired_blacklisted (self, mac))
 		goto ignore;
 
 	if (get_plugin (self, NM_SYSTEM_CONFIG_INTERFACE_CAP_MODIFY_CONNECTIONS))
@@ -1657,7 +1663,7 @@ best_connection_sort (gconstpointer a, gconstpointer b, gpointer user_data)
 {
 	NMSettingsConnection *ac = (NMSettingsConnection *) a;
 	NMSettingsConnection *bc = (NMSettingsConnection *) b;
-	guint64 ats, bts;
+	guint64 ats = 0, bts = 0;
 
 	if (!ac && bc)
 		return -1;
@@ -1669,8 +1675,8 @@ best_connection_sort (gconstpointer a, gconstpointer b, gpointer user_data)
 	g_assert (ac && bc);
 
 	/* In the future we may use connection priorities in addition to timestamps */
-	ats = nm_settings_connection_get_timestamp (ac);
-	bts = nm_settings_connection_get_timestamp (bc);
+	nm_settings_connection_get_timestamp (ac, &ats);
+	nm_settings_connection_get_timestamp (bc, &bts);
 
 	if (ats < bts)
 		return -1;
@@ -1697,6 +1703,8 @@ get_best_connections (NMConnectionProvider *provider,
 
 	g_hash_table_iter_init (&iter, priv->connections);
 	while (g_hash_table_iter_next (&iter, NULL, (gpointer) &connection)) {
+		guint64 cur_ts = 0;
+
 		if (ctype1 && !nm_connection_is_type (NM_CONNECTION (connection), ctype1))
 			continue;
 		if (ctype2 && !nm_connection_is_type (NM_CONNECTION (connection), ctype2))
@@ -1705,10 +1713,11 @@ get_best_connections (NMConnectionProvider *provider,
 			continue;
 
 		/* Don't bother with a connection that's older than the oldest one in the list */
-		if (   max_requested
-		    && added >= max_requested
-		    && nm_settings_connection_get_timestamp (connection) <= oldest)
-			continue;
+		if (max_requested && added >= max_requested) {
+		    nm_settings_connection_get_timestamp (connection, &cur_ts);
+		    if (cur_ts <= oldest)
+				continue;
+		}
 
 		/* List is sorted with oldest first */
 		sorted = g_slist_insert_sorted_with_data (sorted, connection, best_connection_sort, NULL);
@@ -1720,7 +1729,7 @@ get_best_connections (NMConnectionProvider *provider,
 			added--;
 		}
 
-		oldest = nm_settings_connection_get_timestamp (NM_SETTINGS_CONNECTION (sorted->data));
+		nm_settings_connection_get_timestamp (NM_SETTINGS_CONNECTION (sorted->data), &oldest);
 	}
 
 	return g_slist_reverse (sorted);
@@ -1744,6 +1753,14 @@ get_connections (NMConnectionProvider *provider)
 	return g_slist_reverse (list);
 }
 
+static gboolean
+has_connections_loaded (NMConnectionProvider *provider)
+{
+	NMSettingsPrivate *priv = NM_SETTINGS_GET_PRIVATE (provider);
+
+	return priv->connections_loaded;
+}
+
 /***************************************************************/
 
 NMSettings *
@@ -1764,12 +1781,10 @@ nm_settings_new (const char *config_file,
 	priv->dbus_mgr = nm_dbus_manager_get ();
 	priv->bus = nm_dbus_manager_get_connection (priv->dbus_mgr);
 
-	if (plugins) {
-		/* Load the plugins; fail if a plugin is not found. */
-		if (!load_plugins (self, plugins, error)) {
-			g_object_unref (self);
-			return NULL;
-		}
+	/* Load the plugins; fail if a plugin is not found. */
+	if (!load_plugins (self, plugins, error)) {
+		g_object_unref (self);
+		return NULL;
 	}
 
 	unmanaged_specs_changed (NULL, self);
@@ -1783,6 +1798,8 @@ connection_provider_init (NMConnectionProvider *cp_class)
 {
     cp_class->get_best_connections = get_best_connections;
     cp_class->get_connections = get_connections;
+    cp_class->has_connections_loaded = has_connections_loaded;
+    cp_class->add_connection = _nm_connection_provider_add_connection;
 }
 
 static void
